#!/bin/bash

# number of transactions to run on each client
BRANCH=$1
XACTS=$2
RUNS=$3
DBNAME=test


function run_script {

	run=$1
	clients=$2
	sleep=$3
	xacts=$4
	script=$5
	pid=""

	dropdb --if-exists $DBNAME
	createdb $DBNAME

	# build the schema
	psql $DBNAME < tmp.sql >> debug.log 2>&1
	psql $DBNAME < function.sql >> debug.log 2>&1

	psql $DBNAME -c "checkpoint" >> debug.log 2>&1

	psql $DBNAME -c "truncate t" >> debug.log 2>&1
	psql $DBNAME -c "select pg_drop_replication_slot('test_slot')" >> debug.log 2>&1
	psql $DBNAME -c "select pg_create_logical_replication_slot('test_slot', 'test_decoding');" >> debug.log 2>&1

	if [ "$sleep" == "0" ]; then
		sed 's/%SLEEP%//' $script > pgbench.sql 2>&1
	else
		sed "s/%SLEEP%/SELECT pg_sleep($sleep);/" $script > pgbench.sql 2>&1
	fi

	pgbench -n -c $clients -f pgbench.sql -t $((xacts/clients)) $DBNAME > pgbench-${script/.sql/.log} 2>&1

	psql -t -A $DBNAME < changes-master.sql > ${script/.sql/.log} 2>&1 &

	sleep 1

#	rm -f perf.data*
#	pid=$(pgrep -f SELECT)
#
#	if [ "$pid" != "" ]; then
#		perf record -g -p `pgrep -f SELECT`
#		perf report > ${script/.sql/.report}
#	fi

	wait

	# to profile using perf, do this instead:
	#
	# psql -t -A $DBNAME < changes.sql > select4.log 2>&1 &
	#
	# wait for backend to start
	# sleep 1
	#
	# perf record -g -p `pgrep -f SELECT`
	# perf report > ${script/.sql/report}

	timing=`tail -n 1 ${script/.sql/.log} | awk '{print $2}'`

	echo $BRANCH $run $clients $xacts $sleep $script $timing
}

rm -f *.log *.report

# basic table / sequence
echo "begin;
drop table if exists t;
drop sequence if exists s;
create table t (a int);
create sequence s;" > tmp.sql

# sequences used in the alter commands
for i in `seq 1 1000`; do
	echo "drop sequence if exists s$i;
create sequence s$i;" >> tmp.sql
done

echo "commit;" >> tmp.sql

# 5 runs
for r in `seq 1 $RUNS`; do

	# client counts
	for c in 32 96; do

		xacts=$((XACTS/$c))

		for sleep in 0 0.001; do

			# simple sequence nextval
			run_script $r $c $sleep $XACTS nextval.sql

			# simple sequence nextval, but 40 values to trigger WAL-logging
			run_script $r $c $sleep $XACTS nextval-40.sql

			# subtransactions doing create table 
			run_script $r $c $sleep $XACTS create-table.sql

			# subtransactions doing alter sequence, nextval inside
			run_script $r $c $sleep $XACTS alter-sequence.sql

			# subtransactions doing alter sequence, nextval outside
			run_script $r $c $sleep $XACTS alter-sequence-2.sql

			# simple sequence nextval, rollback
			run_script $r $c $sleep $XACTS nextval-abort.sql

			# simple sequence nextval, but 40 values to trigger WAL-logging, rollback
			run_script $r $c $sleep $XACTS nextval-40-abort.sql

			# subtransactions doing create table, rollback
			run_script $r $c $sleep $XACTS create-table-abort.sql

			# subtransactions doing alter sequence, rollback
			run_script $r $c $sleep $XACTS alter-sequence-abort.sql

			# subtransactions doing alter sequence, nextval outside, rollback
			run_script $r $c $sleep $XACTS alter-sequence-2-abort.sql

		done

	done

done
